###############################################################################
#
#  Script name                  : ComDefs.py
#
#  Dependencies                 : None
#                    
#  Objective                    : Common definitions and classes for the 
#                                 BladeCenter update utilities
#                    
#  Written by                   : Mike Nolterieke
#                                 mikenol@us.ibm.com
#                     
#  Change History               : who -   when   - what
#                                 MHN - 07/01/04 - Initial revision.
#  
#  Copyright IBM Corporation, 2004
#
###############################################################################
"""
    Common definitions and classes for the BladeCenter update utilities
"""
# The version of all of the scripts
VERSION = "2.0"

# The product name
PRODUCT = "UpdateXpress Firmware Update Scripts for BladeCenter"

# True/False definitions to support Python versions < 2.3
TRUE = 1
FALSE = 0

# Possible status messages
MM_CMD_STATUS = {0:'PASSED', 1:'FAILED', 2:'UNSUPPORTED',
                 3:'INVALID', 4:'I2C ERROR', 5:'DATA LEN WRONG',
                 6:'DEVICE BUSY', 7:'DEVICE UNAVAILABLE'}

# Carriage return-linefeed characters used during the telnet sessions
CRLF =   "\r\n"

# The prompts to expect during telnet sessions
TNPROMPTS = ["username: ", "password: ", "system> "]

# The prompts to expect during FTP sessions
FTPRPOMPTS = ["Name ", "Password: ", "ftp> "]

# The current lowest supported midplane level
SUPPORTED_MIDPLANE_LEVEL = 5

# The current lowest supported MM code level
SUPPORTED_MM_CODELEVEL = 57

# The MM code level that added support for redundant MMs
REDUNDANT_MM_CODELEVEL = 52

# The maximum number of times to try reconnecting to the MM
MAX_MM_RETRIES = 12

# The maximum number of times to try resending a dot command
MAX_MM_SEND_RETRIES = 10

# The maximum number of times to try reconnecting to the first I/O Module
MAX_IO_RETRIES_FIRST = 12

# The maximum number of times to try reconnecting to each additional I/O Modules
MAX_IO_RETRIES = 6

# Time in seconds to wait for telnet response (long)
TN_TIMEOUT_LONG = 90

# Time in seconds to wait for telnet response (short)
TN_TIMEOUT = 60

# Blade operating system constants
BLADE_OS_LINUX = "Linux"
BLADE_OS_WINDOWS = "Windows"

# Error level defines
WARNING                         = -2
GENERAL_UPDATE_ERROR            = -1
SUCCESS                         = 0
TN_BAD_IP                       = 1
TN_SOCKET_ERR                   = 2
TN_INVALID_RESPONSE             = 3
TN_INVALID_LOGIN                = 4
MM_BAD_IP                       = 5
MM_SOCKET_ERR                   = 6
MM_UNKNOWN_ERR                  = 7
MM_INVALID_KEYS                 = 8
MM_INVALID_LOGIN                = 9
MM_INVALID_RESPONSE             = 10
MM_RETRY_LIMIT_EXCEEDED         = 11
MM_UNKNOWN_CMD_TYPE             = 12
MM_LENGTH_MISMATCH              = 13
MM_COMMAND_MISMATCH             = 14
MM_SEND_FAILURE                 = 15
MM_INVALID_SLOT_NUM             = 16
MM_INVALID_CODE_LEVEL           = 17
MM_PACKET_FILE_NOT_FOUND        = 18
MM_INVALID_PACKET_FILE          = 19
MM_INVALID_PACKET_FILE_HEADER   = 20
TF_DOWNLOAD_TIMEOUT             = 21
TF_SERVER_NOT_FOUND             = 22
TF_LOST_CONNECTION              = 23
TF_INVALID_IP                   = 24
IO_FAILED_TO_CONNECT            = 25
IO_FIRMWARE_IMAGE_NOT_FOUND     = 26
IO_INVALID_IMAGE_FILE           = 27
FTP_BAD_IP                      = 28
FTP_SOCKET_ERR                  = 29
BLADE_UPDATE_GENERAL_ERR        = 30
FTP_INVALID_LOGIN               = 31
MM_SOCKET_TIMEOUT               = 32


# Standard error messages
GENERIC_ERROR = "An error occurred while flashing the management module.  Please consult log file for details\n"
MM_FLASH_SUCCESS = "All Management Module flashes completed successfully!"
IO_WAIT_MESSAGE = "Waiting for the I/O Modules to come up.\nThis will take approximately %s seconds..."
NO_BACKLEVEL = "Error! The Management Module firmware update files are older than\nthe current firmware.  Please specify newer update files."
MM_LEVEL_VERIFIED = "The firmware level is verified to be the same as the update file."

ERROR_LOOKUP = {
    -1:"General update failure.",
    0:"SUCCESS.",
    1:"The supplied I/O module IP address is invalid.",
    2:"The connection to the I/O module was refused or timed out.",
    3:"Too many users are connected to the I/O module.  Cannot connect.",
    4:"The login credentials for the I/O module are invalid.  Verify username and/or password.",
    5:"The supplied Management Module IP address or hostname is invalid.",
    6:"The connection to the Management Module was refused or timed out.",
    7:"An unknown error occurred while communicating with the Management Module.",
    8:"The authentication keys required to communicate with the Management Module are invalid.",
    9:"The login credentials for the Management Module are invalid.  Verify username and/or password.",
    10:"An invalid response was recieved from the Management Module.",
    11:"The limit of retries was exceeded when sending a command the Management Module",
    12:"An unknown command type for the Management Module was specified",
    13:"The length of the Management Module command and the response do not match",
    14:"The command sent to and the command recieved back from the Management Module to not match",
    15:"There was an error sending data to the Management Module",
    16:"An invalid slot number for the BladeCenter Chassis was specified",
    17:"The current code level of the Management Module is not supported by this utility.",
    18:"One or more files required to flash the Management Module were not found.",
    19:"One or more files required to flash the Management Module were invalid.",
    20:"One or more files required to flash the Management Module contained an invalid header.",
    21:"A timeout occurred while attempting to copy the update file(s) from the TFTP server.",
    22:"The TFTP server was not found.",
    23:"The connection to the TFTP server was unexpectedly terminated.",
    24:"An invalid TFTP server address was specified.",
    25:"Failed to connect to the I/O module.",
    26:"Unable to locate the I/O module firmware image on the TFTP server.",
    27:"The I/O module firmware image is invalid.",
    28:"The FTP server IP address is invalid.",
    29:"A socket error occurred while attempting to connect to the FTP server.",
    30:"An unknown error occurred while updating one more more blades.",
    31:"The login credentials for the FTP server are invalid.  Verify username and/or password."
}

    
def GetLastError(errorCode):
    global ERROR_LOOKUP
    for error, message in ERROR_LOOKUP.iteritems():
        if errorCode == error:
            return "ERROR %s! %s" % (error, message)

               
class parameters:
    '''All of the required paramters for updating the MM and I/O Modules of a BladeCenter chassis'''
    mmIPAddress = ""
    # Default to USERID and PASSW0RD
    mmUserName = ""
    mmPassword = ""
    mmFilename1 = ""
    mmFilename2 = ""
    mmFilename3 = ""
    mmdisable = FALSE
    localFileLocation = "."
    # We are good to go if the MM and all of the I/O modules use the same user id and password
    io1UserName = ""
    io1Password = ""
    io1disable  = FALSE
    io2UserName = ""
    io2Password = ""
    io2disable  = FALSE
    io3UserName = ""
    io3Password = ""
    io3disable  = FALSE
    io4UserName = ""
    io4Password = ""
    io4disable  = FALSE
    io1Filename1 = ""
    io1Filename2 = ""
    io2Filename1 = ""
    io2Filename2 = ""
    io3Filename1 = ""
    io3Filename2 = ""
    io4Filename1 = ""
    io4Filename2 = ""
    # If set to 1 will spit out debug messages
    debugLevel = 0
    responseFile = ""
    continueIO = FALSE
    forceMMreboot = FALSE
    UXCDLocation = "."
    tftpIPAddr = ""
    ftpIPAddress = ""
    ftpUserName = ""
    ftpPassword = ""
    
    # Blade parameters
    blade1name = ""
    blade1user = ""
    blade1pass = ""
    blade1ipaddr = ""
    blade1os = ""
    blade1disable = FALSE
    blade1reboot = FALSE
    
    blade2name = ""
    blade2user = ""
    blade2pass = ""
    blade2ipaddr = ""
    blade2os = ""
    blade2disable = FALSE
    blade2reboot = FALSE
    
    blade3name = ""
    blade3user = ""
    blade3pass = ""
    blade3ipaddr = ""
    blade3os = ""
    blade3disable = FALSE
    blade3reboot = FALSE
    
    blade4name = ""
    blade4user = ""
    blade4pass = ""
    blade4ipaddr = ""
    blade4os = ""
    blade4disable = FALSE
    blade4reboot = FALSE
    
    blade5name = ""
    blade5user = ""
    blade5pass = ""
    blade5ipaddr = ""
    blade5os = ""
    blade5disable = FALSE
    blade5reboot = FALSE
    
    blade6name = ""
    blade6user = ""
    blade6pass = ""
    blade6ipaddr = ""
    blade6os = ""
    blade6disable = FALSE
    blade6reboot = FALSE
    
    blade7name = ""
    blade7user = ""
    blade7pass = ""
    blade7ipaddr = ""
    blade7os = ""
    blade7disable = FALSE
    blade7reboot = FALSE
    
    blade8name = ""
    blade8user = ""
    blade8pass = ""
    blade8ipaddr = ""
    blade8os = ""
    blade8disable = FALSE
    blade8reboot = FALSE
    
    blade9name = ""
    blade9user = ""
    blade9pass = ""
    blade9ipaddr = ""
    blade9os = ""
    blade9disable = FALSE
    blade9reboot = FALSE
    
    blade10name = ""
    blade10user = ""
    blade10pass = ""
    blade10ipaddr = ""
    blade10os = ""
    blade10disable = FALSE
    blade10reboot = FALSE
    
    blade11name = ""
    blade11user = ""
    blade11pass = ""
    blade11ipaddr = ""
    blade11os = ""
    blade11disable = FALSE
    blade11reboot = FALSE
    
    blade12name = ""
    blade12user = ""
    blade12pass = ""
    blade12ipaddr = ""
    blade12os = ""
    blade12disable = FALSE
    blade12reboot = FALSE
    
    blade13name = ""
    blade13user = ""
    blade13pass = ""
    blade13ipaddr = ""
    blade13os = ""
    blade13disable = FALSE
    blade13reboot = FALSE
    
    blade14name = ""
    blade14user = ""
    blade14pass = ""
    blade14ipaddr = ""
    blade14os = ""
    blade14disable = FALSE
    blade14reboot = FALSE
    
    def print_params(self):
        print "###  Current Parameter Values  ###"
        print "mmIPAddress: %s" % self.mmIPAddress
        print "mmUserName: %s" % self.mmUserName
        print "mmPassword: %s" % self.mmPassword
        print "mmdisable: %s" % self.mmdisable
        print "tftpIPAddr: %s" % self.tftpIPAddr
        print "io1UserName: %s" % self.io1UserName
        print "io1Password: %s" % self.io1Password
        print "io1disable: %s" % self.io1disable
        print "io2UserName: %s" % self.io2UserName
        print "io2Password: %s" % self.io2Password
        print "io2disable: %s" % self.io2disable
        print "io3UserName: %s" % self.io3UserName
        print "io3Password: %s" % self.io3Password
        print "io3disable: %s" % self.io3disable
        print "io4UserName: %s" % self.io4UserName
        print "io4Password: %s" % self.io4Password
        print "io4disable: %s" % self.io4disable
        print "io1Filename1: %s" % self.io1Filename1
        print "io1Filename2: %s" % self.io1Filename2
        print "io2Filename1: %s" % self.io2Filename1
        print "io2Filename2: %s" % self.io2Filename2
        print "io3Filename1: %s" % self.io3Filename1
        print "io3Filename2: %s" % self.io3Filename2
        print "io4Filename1: %s" % self.io4Filename1
        print "io4Filename2: %s" % self.io4Filename2
        print "debugLevel: %s" % self.debugLevel
        print "responseFile: %s" % self.responseFile
        print "continueIO: %s" % self.continueIO
        print "UXCDLocation: %s" % self.UXCDLocation
        print "ftpIPAddress: %s" % self.ftpIPAddress
        print "ftpUserName: %s" % self.ftpUserName
        print "ftpPassword: %s" % self.ftpPassword
