#! /usr/bin/env python
###############################################################################
#
#  Script name                  : ComFuncs.py
#
#  Dependencies                 : ComDefs.py
#                    
#  Objective                    : Common functions/utilities for the BladeCenter updates
#                    
#  Written by                   : Mike Nolterieke
#                                 mikenol@us.ibm.com
#                     
#  Change History               : who -   when   - what
#                                 MHN - 07/01/04 - Initial revision.
#  
#  Copyright IBM Corporation, 2004
#
###############################################################################
"""
Common functions/utilities for the BladeCenter updates
"""
try:
    import telnetlib, sys, string, os
    from socket import *
    from ftplib import FTP, error_perm, error_proto, error_reply, error_temp
    sys.path.append("..")
    import ComDefs
except KeyboardInterrupt:
    sys.exit(1)

def intToHex(intNum, isString):
    """ Takes an int number argument (only 1 - 14 for now) and returns the proper hex
    value string or hex number depending on the isString argument
    For Example: We will return '\\x0A' if intNum = 10 and isString = True
                 We will return 0x0B if intNum = 11 and isString = False
    Returns:
        The hex value for the slot number as a string or number
    """
    # The hex number to return
    # Possible slots are 1 - 14
    hexNum = ""
    if intNum == 1:
        if isString:
            hexNum = "\x01"
        else:
            hexNum = 0x01
    elif intNum == 2:
        if isString:
            hexNum = "\x02"
        else:
            hexNum = 0x02
    elif intNum == 3:
        if isString:
            hexNum = "\x03"
        else:
            hexNum = 0x03
    elif intNum == 4:
        if isString:
            hexNum = "\x04"
        else:
            hexNum = 0x04
    elif intNum == 5:
        if isString:
            hexNum = "\x05"
        else:
            hexNum = 0x05
    elif intNum == 6:
        if isString:
            hexNum = "\x06"
        else:
            hexNum = 0x06
    elif intNum == 7:
        if isString:
            hexNum = "\x07"
        else:
            hexNum = 0x07
    elif intNum == 8:
        if isString:
            hexNum = "\x08"
        else:
            hexNum = 0x08
    elif intNum == 9:
        if isString:
            hexNum = "\x09"
        else:
            hexNum = 0x09
    elif intNum == 10:
        if isString:
            hexNum = "\x0A"
        else:
            hexNum = 0x0A
    elif intNum == 11:
        if isString:
            hexNum = "\x0B"
        else:
            hexNum = 0x0B
    elif intNum == 12:
        if isString:
            hexNum = "\x0C"
        else:
            hexNum = 0x0C
    elif intNum == 13:
        if isString:
            hexNum = "\x0D"
        else:
            hexNum = 0x0D
    elif intNum == 14:
        if isString:
            hexNum = "\x0E"
        else:
            hexNum = 0x0E
    return hexNum

def connectTelnet(ipAddress, userName, password, debugLevel=0):
    """Connect to the telnet interface of the specified IP address using the specfied
    user name and password.  You may also specify the debug_level (defaults to 0).
    It then returns the connected telnet object or a predefined error level."""
    # Connect to the IP address
    try:
        tn = telnetlib.Telnet(ipAddress)
    # Check for errors
    except gaierror:
        return ComDefs.TN_BAD_IP
    except error:
        return ComDefs.TN_SOCKET_ERR
    # Set the debug level
    tn.set_debuglevel(debugLevel)
    # Give it 5 seconds to to get one of the prompts we expect
    if tn.expect(ComDefs.TNPROMPTS, 5)[0] == -1:
        return ComDefs.TN_INVALID_RESPONSE
    # Send the user name
    tn.write(userName + ComDefs.CRLF)
    # Give it 5 seconds to to get one of the prompts we expect
    if tn.expect(ComDefs.TNPROMPTS, 5)[0] == -1:
        return ComDefs.TN_INVALID_RESPONSE
    # Send the password
    tn.write(password + ComDefs.CRLF)
    # Return the connected telnet object
    return tn

def disconnectTelnet(tn):
    """ Disconnect connected telnet sessions"""
    tn.close()

    
def connectFtp(ipAddress, userName, password, debugLevel=0):
    """
        Connect to the FTP server at the specified IP address 
        using the specified username/password.  Returns connected
        FTP object.
    """
    
    try:
        #ftp = ftplib.FTP(ipAddress, userName, password)
        addr = gethostbyname('9.42.215.145')
        ftp = FTP()
        setdefaulttimeout(60)
        ftp.connect(addr)
        ftp.login(userName, password)
    except gaierror:
        return ComDefs.FTP_BAD_IP
    except error:
        return ComDefs.FTP_SOCKET_ERR
    except (error_perm, error_temp, error_reply, error_proto):
        print ("Oops, couldn't connect")
        return ComDefs.GENERAL_UPDATE_ERROR
        
    ftp.set_debuglevel(debugLevel)
    
    
    return ftp
    
def disconnectFtp(ftp):
    ftp.close()


def keyboardISR(logger):
    if logger:
        logger.log("Keyboard interrupt recieved.  Process halted by user.")
    sys.exit()


def getBuildID(buildfile):

    empty = "       "
    
    try:
        input = open(buildfile, 'r')
    except:
        print "\nCannot open BUILDID file (%s) for reading." % buildfile
        return empty

    if input == None:
        print "\nBUILDID file (%s) is empty." % buildfile
        return empty

    line = input.read(9)
    input.close()

    return line 
